Package: RcppMeCab
Title: 'rcpp' Wrapper for 'mecab' Library
Version: 0.0.1.5
Authors@R: c(person("Junhewk", "Kim", role = c("aut", "cre"),
                    email = "junhewk.kim@gmail.com"),
             person("Taku", "Kudo", role = c("aut"),
                    email = "taku@chasen.org"),
             person("Akiru", "Kato", role = c("ctb")),
             person("Patrick", "Schratz", role = c("ctb")))
Description: R package based on 'Rcpp' for 'MeCab': Yet Another Part-of-Speech and Morphological Analyzer. 
    The purpose of this package is providing a seamless developing and analyzing environment for CJK texts.
    This package utilizes parallel programming for providing highly efficient text preprocessing 'posParallel()' function.
    For installation, please refer to README.md file.
Depends: R (>= 3.4.0)
License: GPL
Encoding: UTF-8
BugReports: https://github.com/junhewk/RcppMeCab/issues
RoxygenNote: 7.3.3
Language: en-US
LinkingTo: Rcpp, RcppParallel, BH
Imports: Rcpp, RcppParallel
Suggests: testthat, spelling
SystemRequirements: MeCab 0.996 or higher for Japanese (libmecab-dev
        (deb), mecab-devel (rpm)), mecab-ko 0.999
        (https://github.com/Pusnow/mecab-ko-msvc) for Korean
NeedsCompilation: yes
Packaged: 2026-03-24 05:41:08 UTC; jk
Author: Junhewk Kim [aut, cre],
  Taku Kudo [aut],
  Akiru Kato [ctb],
  Patrick Schratz [ctb]
Maintainer: Junhewk Kim <junhewk.kim@gmail.com>
Repository: CRAN
Date/Publication: 2026-03-24 06:00:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 21:39:03 UTC; unix
Archs: RcppMeCab.so.dSYM

Package: RcppMeCab
Title: 'rcpp' Wrapper for 'mecab' Library
Version: 0.0.1.7
Authors@R: c(person("Junhewk", "Kim", role = c("aut", "cre"),
                    email = "junhewk.kim@gmail.com"),
             person("Taku", "Kudo", role = c("aut"),
                    email = "taku@chasen.org"),
             person("Akiru", "Kato", role = c("ctb")),
             person("Patrick", "Schratz", role = c("ctb")))
Description: R package based on 'Rcpp' for 'MeCab': Yet Another Part-of-Speech and Morphological Analyzer.
    It provides install-time engine profiles and dictionaries for Japanese,
    Korean, and Mandarin Chinese text. Runtime dictionary selection does not
    change the installed engine.
    This package utilizes parallel programming for providing highly efficient text preprocessing 'posParallel()' function.
    For installation, please refer to README.md file.
Depends: R (>= 3.4.0)
License: GPL
Encoding: UTF-8
BugReports: https://github.com/junhewk/RcppMeCab/issues
RoxygenNote: 7.3.3
Language: en-US
Config/roxygen2/version: 8.0.0
LinkingTo: Rcpp, RcppParallel, BH
Imports: Rcpp, RcppParallel
Suggests: testthat, spelling
SystemRequirements: MeCab 0.996 or higher for Japanese and Chinese
        (libmecab-dev (deb), mecab-devel (rpm)), mecab-jieba 0.1.1 for
        Mandarin Chinese (https://github.com/lindera/mecab-jieba),
        mecab-ko 0.999 (https://github.com/Pusnow/mecab-ko-msvc) for
        Korean
NeedsCompilation: yes
Packaged: 2026-07-13 02:44:26 UTC; jk
Author: Junhewk Kim [aut, cre],
  Taku Kudo [aut],
  Akiru Kato [ctb],
  Patrick Schratz [ctb]
Maintainer: Junhewk Kim <junhewk.kim@gmail.com>
Repository: CRAN
Date/Publication: 2026-07-13 09:00:02 UTC

# Generated by roxygen2: do not edit by hand

export(dict_index)
export(download_dic)
export(list_dic)
export(pos)
export(posParallel)
export(set_dic)
import(Rcpp)
importFrom(Rcpp,sourceCpp)
importFrom(RcppParallel,RcppParallelLibs)
useDynLib(RcppMeCab, .registration=TRUE)

# RcppMeCab

[![License](https://img.shields.io/badge/license-GPL-brightgreen.svg?style=flat)](http://www.gnu.org/licenses/gpl.html)
![R](https://img.shields.io/github/r-package/v/junhewk/RcppMeCab)
[![CRAN](https://www.r-pkg.org/badges/version/RcppMeCab)](https://cran.r-project.org/package=RcppMeCab)
[![Downloads](https://cranlogs.r-pkg.org/badges/RcppMeCab?color=brightgreen)](https://www.r-pkg.org/pkg/RcppMeCab)
[![R-CMD-check](https://github.com/junhewk/RcppMeCab/actions/workflows/R-CMD-check.yml/badge.svg)](https://github.com/junhewk/RcppMeCab/actions/workflows/R-CMD-check.yml)

This package, RcppMeCab, is a `Rcpp` wrapper for the part-of-speech morphological analyzer `MeCab`. It supports native utf-8 encoding in C++ code and CJK (Chinese, Japanese, and Korean) MeCab library. This package fully utilizes the power `Rcpp` brings `R` computation to analyze texts faster.

__Please see [this](https://github.com/junhewk/RcppMeCab/blob/master/README_kr.md) for easy installation and usage examples in Korean.__

## MeCab backends

RcppMeCab builds MeCab from source at install time. The MeCab variant is selected by the `MECAB_LANG` environment variable:

| `MECAB_LANG` | Backend | Version | Source |
|---|---|---|---|
| `ko` (default) | [mecab-ko-msvc](https://github.com/Pusnow/mecab-ko-msvc) | 0.999 | Pusnow/mecab-ko-msvc |
| `ja` | [MeCab](http://taku910.github.io/mecab/) | 0.996 | taku910/mecab |

On Linux and macOS, if MeCab is already installed system-wide (detected via `mecab-config`), RcppMeCab uses the system installation regardless of `MECAB_LANG`.

## Installation

### Linux, macOS, and Windows

RcppMeCab automatically downloads and builds MeCab from source if it is not already installed on your system. No manual MeCab installation is required.

```r
install.packages("RcppMeCab") # install from CRAN

# or install the development version
# install.packages("devtools")
devtools::install_github("junhewk/RcppMeCab")
```

If you already have MeCab installed (e.g. via `brew install mecab` on macOS, or `apt install libmecab-dev` on Linux), RcppMeCab will use your system installation.

### Language selection

Set `MECAB_LANG` before installation to choose the MeCab variant:

```r
# Korean (default)
install.packages("RcppMeCab", type = "source")

# Japanese
Sys.setenv(MECAB_LANG = "ja")
install.packages("RcppMeCab", type = "source")
```

### Dictionary

A MeCab dictionary is **automatically downloaded and installed** during package installation:

+ **Korean** (`MECAB_LANG=ko`, default): [mecab-ko-dic](https://github.com/Pusnow/mecab-ko-msvc/releases) (pre-compiled, from mecab-ko-msvc releases)
+ **Japanese** (`MECAB_LANG=ja`): [IPAdic](http://taku910.github.io/mecab/) (compiled from source during installation)

The bundled dictionary is stored in the package's `dic/` directory and used automatically — no manual dictionary setup is required.

### Downloading additional dictionaries

You can download and install dictionaries for other languages after installation using `download_dic()`. No system-level MeCab installation is required — dictionary compilation is handled entirely within R.

```r
download_dic("ja") # download and compile Japanese IPAdic
download_dic("ko") # download Korean mecab-ko-dic
download_dic("zh") # download and compile Chinese mecab-jieba
```

Dictionaries are stored in the user data directory (`tools::R_user_dir("RcppMeCab", "data")`) and persist across R sessions.

Use `list_dic()` to see all installed dictionaries:

```r
list_dic()
#>      lang         name                              path active
#> 1 bundled      bundled /path/to/RcppMeCab/dic              TRUE
#> 2      ja       ipadic ~/.local/share/R/RcppMeCab/ja      FALSE
#> 3      ko mecab-ko-dic ~/.local/share/R/RcppMeCab/ko      FALSE
#> 4      zh  mecab-jieba ~/.local/share/R/RcppMeCab/zh      FALSE
```

## Usage

This package has `pos` and `posParallel` functions.

```r
pos(sentence)                        # returns a list
pos(sentence, join = FALSE)          # morphemes only (tags as vector names)
pos(sentence, format = "data.frame") # returns a data frame
pos(sentence, user_dic = "path")     # with a compiled user dictionary
posParallel(sentence)                # parallelized, faster for large inputs
```

### Switching languages

Use the `lang` parameter to select a dictionary by language:

```r
pos("東京は日本の首都です。", lang = "ja")
pos("안녕하세요", lang = "ko")
pos("我是中国人。", lang = "zh")
```

Or set a default with `set_dic()`:

```r
set_dic("ja")
pos("東京は日本の首都です。") # uses Japanese dictionary
set_dic("ko")
pos("안녕하세요")              # uses Korean dictionary
set_dic("bundled")             # switch back to the build-time dictionary
```

You can also specify a custom dictionary path directly:

```r
pos("text", sys_dic = "/path/to/custom-dic")
options(mecabSysDic = "/path/to/custom-dic")
```

### Parameters

+ `sentence`: text to analyze
+ `join`: if `TRUE` (default), output is `morpheme/tag`; if `FALSE`, output is `morpheme` with tag as attribute
+ `format`: `"list"` (default) or `"data.frame"`
+ `lang`: language code (`"ja"`, `"ko"`, or `"zh"`) to select a dictionary installed via `download_dic()`. Overrides `sys_dic` when specified.
+ `sys_dic`: directory containing `dicrc`, `sys.dic`, etc. Set a default with `options(mecabSysDic = "/path/to/dic")`
+ `user_dic`: path to a user dictionary compiled by `dict_index()`

Note: provide full paths for `sys_dic` and `user_dic` (no tilde `~/` expansion).

## Compiling a user dictionary

RcppMeCab provides the `dict_index()` function to compile user dictionaries directly from R, without needing the `mecab-dict-index` command-line tool.

Prepare your entries as a CSV file ([Japanese format](http://taku910.github.io/mecab/dic.html), [Korean format](https://bitbucket.org/eunjeon/mecab-ko-dic/src/master/final/user-dic/README.md)), then compile:

```r
dict_index(
  dic_csv = "entries.csv",
  out_dic = "userdic.dic",
  dic_dir = "/path/to/mecab-dic"
)

# Then use the compiled dictionary:
pos("some text", user_dic = "userdic.dic")
```

## Authors

Junhewk Kim (junhewk.kim@gmail.com), Taku Kudo

## Contributors

Akiru Kato, Patrick Schratz

# RcppMeCab

[![R-CMD-check](https://github.com/junhewk/RcppMeCab/actions/workflows/R-CMD-check.yml/badge.svg?branch=master)](https://github.com/junhewk/RcppMeCab/actions/workflows/R-CMD-check.yml)
[![CRAN status](https://www.r-pkg.org/badges/version/RcppMeCab)](https://CRAN.R-project.org/package=RcppMeCab)
[![CRAN downloads](https://cranlogs.r-pkg.org/badges/RcppMeCab)](https://CRAN.R-project.org/package=RcppMeCab)
[![CRAN total downloads](https://cranlogs.r-pkg.org/badges/grand-total/RcppMeCab)](https://CRAN.R-project.org/package=RcppMeCab)
[![R version](https://img.shields.io/badge/R-%E2%89%A5%203.4.0-276DC3.svg)](https://www.r-project.org/)
[![License: GPL](https://img.shields.io/badge/license-GPL-blue.svg)](https://www.gnu.org/licenses/gpl-3.0.html)

RcppMeCab is an `Rcpp` wrapper for the MeCab part-of-speech morphological
analyzer. It provides UTF-8 installation profiles and dictionaries for
Japanese, Korean, and Mandarin Chinese text. Engine selection happens at
package installation; runtime dictionary selection does not replace that
engine. Rcpp and parallel processing provide efficient text analysis.

__Please see [this](https://github.com/junhewk/RcppMeCab/blob/master/README_kr.md) for easy installation and usage examples in Korean.__

## Build profiles and MeCab engines

When a system MeCab installation is not found, RcppMeCab builds an engine from
source. `MECAB_LANG` selects that source-build profile and its bundled
dictionary:

| `MECAB_LANG` | Engine | Version | Bundled dictionary |
|---|---|---|---|
| `ko` (default) | [mecab-ko-msvc](https://github.com/Pusnow/mecab-ko-msvc) | 0.999 | mecab-ko-dic |
| `ja` | [MeCab](https://taku910.github.io/mecab/) | 0.996 | IPAdic |
| `zh` | [MeCab](https://taku910.github.io/mecab/) | 0.996 | [mecab-jieba 0.1.1](https://github.com/lindera/mecab-jieba) |

MeCab is dictionary-driven, so Chinese uses the standard MeCab engine with a
Mandarin dictionary rather than a separate Chinese engine. The Chinese profile
supports simplified and traditional UTF-8 text.

For the `ja` and `ko` profiles on Linux and macOS, an existing system MeCab
installation (detected via `mecab-config`) takes precedence. The `zh` profile
always builds the pinned standard MeCab engine so its compiler and
mecab-jieba dictionary remain compatible. `MECAB_LANG` is an installation
setting; it does not switch engines in a running R session.

## Installation

### Linux, macOS, and Windows

RcppMeCab automatically downloads and builds MeCab from source if it is not already installed on your system. No manual MeCab installation is required.

```r
install.packages("RcppMeCab") # install from CRAN

# or install the development version
# install.packages("devtools")
devtools::install_github("junhewk/RcppMeCab")
```

If you already have MeCab installed (e.g. via `brew install mecab` on macOS, or `apt install libmecab-dev` on Linux), RcppMeCab will use your system installation.

### Installation profile

Set `MECAB_LANG` before installation to choose the engine and bundled
dictionary profile. Existing system engines take precedence for `ja` and `ko`
as described above; `zh` always uses the pinned source build.

```r
# Korean (default)
install.packages("RcppMeCab", type = "source")

# Japanese
Sys.setenv(MECAB_LANG = "ja")
install.packages("RcppMeCab", type = "source")

# Chinese (Mandarin; simplified and traditional text)
Sys.setenv(MECAB_LANG = "zh")
install.packages("RcppMeCab", type = "source")
```

### Dictionary

When RcppMeCab builds its own engine, the matching dictionary is
**automatically downloaded and installed** during package installation:

+ **Korean** (`MECAB_LANG=ko`, default): [mecab-ko-dic](https://github.com/Pusnow/mecab-ko-msvc/releases) (pre-compiled, from mecab-ko-msvc releases)
+ **Japanese** (`MECAB_LANG=ja`): [IPAdic](http://taku910.github.io/mecab/) (compiled from source during installation)
+ **Chinese** (`MECAB_LANG=zh`): [mecab-jieba](https://github.com/lindera/mecab-jieba) (compiled from source during installation)

The bundled dictionary is stored in the package's `dic/` directory and used
automatically. The `ja` and `ko` profiles use a system-configured dictionary
when they reuse a system MeCab installation. The `zh` profile always compiles
and bundles mecab-jieba with its pinned engine.

### Downloading additional dictionaries

You can download and install dictionaries for other languages after installation using `download_dic()`. No system-level MeCab installation is required — dictionary compilation is handled entirely within R.

```r
download_dic("ja") # download and compile Japanese IPAdic
download_dic("ko") # download Korean mecab-ko-dic
download_dic("zh") # download and compile Chinese mecab-jieba
```

Dictionaries are stored in the user data directory (`tools::R_user_dir("RcppMeCab", "data")`) and persist across R sessions.

Use `list_dic()` to see all installed dictionaries:

```r
list_dic()
#>      lang         name                              path active
#> 1 bundled      bundled /path/to/RcppMeCab/dic              TRUE
#> 2      ja       ipadic ~/.local/share/R/RcppMeCab/ja      FALSE
#> 3      ko mecab-ko-dic ~/.local/share/R/RcppMeCab/ko      FALSE
#> 4      zh  mecab-jieba ~/.local/share/R/RcppMeCab/zh      FALSE
```

## Usage

This package has `pos` and `posParallel` functions.

```r
pos(sentence)                        # returns a list
pos(sentence, join = FALSE)          # morphemes only (tags as vector names)
pos(sentence, format = "data.frame") # returns a data frame
pos(sentence, user_dic = "path")     # with a compiled user dictionary
posParallel(sentence)                # parallelized, faster for large inputs
dictionary_info()                    # active dictionary metadata
```

### Selecting dictionaries

`lang` selects a dictionary previously installed with `download_dic()`. It
does not change the MeCab engine compiled or linked into RcppMeCab.

```r
download_dic("ja")
download_dic("zh")

pos("東京は日本の首都です。", lang = "ja")
pos("我是中国人。", lang = "zh")
```

The active default dictionary can also be changed:

```r
set_dic("ja")
pos("東京は日本の首都です。")
set_dic("zh")
pos("我是中国人。")
set_dic("bundled") # restore the build-time dictionary
```

Japanese and Chinese dictionaries use the standard MeCab engine and can be
selected within a standard MeCab installation. Supported Korean analysis
requires the mecab-ko engine. Loading `mecab-ko-dic` into standard MeCab does
not add mecab-ko's Korean-specific whitespace behavior. Conversely, loading a
Japanese or Chinese dictionary does not replace an installed mecab-ko engine.
Use a separate RcppMeCab installation or reinstall with the required engine
profile when moving between Korean and standard MeCab behavior.

Dictionary feature layouts are also language-specific. List output consistently
uses each token's primary POS tag. The historical `data.frame` columns
`subtype` and `analytic` follow the Japanese/Korean layout; they do not expose
all mecab-jieba fields. Pinyin, traditional/simplified forms, and definitions
do not yet have dedicated output columns.

A dictionary can also be selected by path:

```r
pos("text", sys_dic = "/path/to/custom-dic")
options(mecabSysDic = "/path/to/custom-dic")
```

`dictionary_info()` reports the filename, charset, type, size, context sizes,
and version of every dictionary loaded by MeCab. It reports dictionaries, not
the language or engine variant, and is useful for confirming which dictionary
is active.

### Parameters

+ `sentence`: text to analyze
+ `join`: if `TRUE` (default), output is `morpheme/tag`; if `FALSE`, output is `morpheme` with tag as attribute
+ `format`: `"list"` (default) or `"data.frame"`
+ `lang`: dictionary code (`"ja"`, `"ko"`, or `"zh"`) selecting a dictionary installed via `download_dic()`. It does not switch engines and overrides `sys_dic` when specified.
+ `sys_dic`: directory containing `dicrc`, `sys.dic`, etc. Set a default with `options(mecabSysDic = "/path/to/dic")`
+ `user_dic`: path to a user dictionary compiled by `dict_index()`

Note: provide full paths for `sys_dic` and `user_dic` (no tilde `~/` expansion).

## Compiling a user dictionary

RcppMeCab provides the `dict_index()` function to compile user dictionaries directly from R, without needing the `mecab-dict-index` command-line tool.

Prepare your entries as a CSV file ([Japanese format](http://taku910.github.io/mecab/dic.html), [Korean format](https://bitbucket.org/eunjeon/mecab-ko-dic/src/master/final/user-dic/README.md)), then compile:

```r
dict_index(
  dic_csv = "entries.csv",
  out_dic = "userdic.dic",
  dic_dir = "/path/to/mecab-dic"
)

# Then use the compiled dictionary:
pos("some text", user_dic = "userdic.dic")
```

## Authors

Junhewk Kim (junhewk.kim@gmail.com), Taku Kudo

## Contributors

Akiru Kato, Patrick Schratz

# 사용자 사전 추가 (Mecab-Ko-MSVC)

## 준비
[mecab-ko-msvc](https://github.com/Pusnow/mecab-ko-msvco)와 [mecab-ko-dic-msvc](https://github.com/Pusnow/mecab-ko-dic-msvc) 을 다운받아 설치합니다. 
반드시 `C:\mecab` 경로에 설치하셔야 사전 추가가 가능합니다.

## 사전 추가
`C:\mecab\user-dic` 디렉토리 안에 csv 확장자로 사전 파일을 추가합니다.


    user-dic/
    ├── nnp.csv
    ├── person.csv
    └── place.csv

* 일반적인 고유명사 추가
    
        대우,,,,NNP,*,F,대우,*,*,*,*
        구글,,,,NNP,*,T,구글,*,*,*,*

* 인명 추가

        까비,,,,NNP,인명,F,까비,*,*,*,*

* 지명 추가

        세종,,,,NNP,지명,T,세종,*,*,*,*
        세종시,,,,NNP,지명,F,세종시,Compound,*,*,세종/NNP/지명+시/NNG/*

그 외의 품사 추가가 필요한 경우에는 [품사태그표](https://docs.google.com/spreadsheet/ccc?key=0ApcJghR6UMXxdEdURGY2YzIwb3dSZ290RFpSaUkzZ0E&usp=sharing#gid=4) 를 참고하세요.

## 사전 빌드

**경고: 이 작업은 사용자 사전의 수정사항(코스트 등)을 초기화합니다. 사용자 사전을 수정했을 경우 사용자 사전 파일(`user-xxx.csv`)를 백업하고 다음 작업을 시행하십시오.**

Powershell을 이용하여 작업하셔야 합니다. 단축키 `(윈도우키) + R` 을 누르고 `powershell` 을 입력하면 실행 가능합니다.

* `C:\mecab` 경로로 이동합니다.

        PS C:\Users> cd C:\mecab
        PS C:\mecab>
* `tools\add-userdic-win.ps1` 스크립트를 실행합니다.

        PS C:\mecab> tools\add-userdic-win.ps1
        generating userdic...
        C:\mecab\mecab-ko-dic\model.def is not a binary model. reopen it as text mode...
        reading C:\mecab\user-dic\nnp.csv ...
        done!
        ...생략

    
아래와 같이 user-xxx.csv 사전이 추가된 모습을 볼 수 있습니다. 사실 아래 파일은 컴파일 되기 직전의 파일이며, 실제로 sys.dic 파일에 바이너리로 컴파일 되어 들어가게 됩니다.

    mecab-ko-dic
    ├── ....
    ├── user-nnp.csv
    ├── user-person.csv
    ├── user-place.csv
    └── ...

## 사전 컴파일

다음 절차를 통해 사용자 사전 파일을 만들지 않고 사전을 컴파일 할 수 있습니다.

사용자 사전 파일(`user-xxx.csv`)을 수동으로 수정했으면 다음 절차로 변경 내용(코스트 등)을 반영할 수 있습니다.

* `C:\mecab` 경로로 이동합니다.
* `tools\compile-win.ps1` 스크립트를 실행합니다.



## 설치
설치 과정은 따로 필요 없습니다. 바로 사용하시면 됩니다.


# 사용자 사전 추가 (Original)

## 준비
[mecab-ko](https://bitbucket.org/eunjeon/mecab-ko)와 [mecab-ko-dic](https://bitbucket.org/eunjeon/mecab-ko-dic) 을 다운받아 설치합니다. mecab-ko-dic 을 꼭 컴파일까지 하셔야 이후에 사전 추가가 가능합니다.

## 사전 추가
내려받은 mecab-ko-dic/userdic 디렉토리 안에 csv 확장자로 사전 파일을 추가합니다.

    :::text
    userdic/
    ├── nnp.csv
    ├── person.csv
    └── place.csv


  * 일반적인 고유명사 추가
    
        :::text
        대우,,,,NNP,*,F,대우,*,*,*,*
        구글,,,,NNP,*,T,구글,*,*,*,*

  * 인명 추가

        :::text
        까비,,,,NNP,인명,F,까비,*,*,*,*
    
  * 지명 추가

        :::text
        세종,,,,NNP,지명,T,세종,*,*,*,*
        세종시,,,,NNP,지명,F,세종시,Compound,*,*,세종/NNP/지명+시/NNG/*

그 외의 품사 추가가 필요한 경우에는 [품사태그표](https://docs.google.com/spreadsheet/ccc?key=0ApcJghR6UMXxdEdURGY2YzIwb3dSZ290RFpSaUkzZ0E&usp=sharing#gid=4) 를 참고하세요.

 
## 사전 컴파일
    :::text
    $ mecab-ko-dic/tools/add-userdic.sh

아래와 같이 user-xxx.csv 사전이 추가된 모습을 볼 수 있습니다. 사실 아래 파일은 컴파일 되기 직전의 파일이며, 실제로 sys.dic 파일에 바이너리로 컴파일 되어 들어가게 됩니다.

    :::text
    mecab-ko-dic
    ├── ....
    ├── user-nnp.csv
    ├── user-person.csv
    ├── user-place.csv
    └── ...

## 설치
    :::text
    $ make install

#' @noRd
## usethis namespace: start
#' @importFrom Rcpp sourceCpp
#' @useDynLib RcppMeCab, .registration=TRUE
## usethis namespace: end
NULL

#' @noRd
.onLoad <- function(libname, pkgname) {
  # Set bundled dictionary as default if no mecabSysDic option is set
  if (is.null(getOption("mecabSysDic"))) {
    dic_dir <- system.file("dic", package = pkgname)
    if (nzchar(dic_dir) && file.exists(file.path(dic_dir, "sys.dic"))) {
      options(mecabSysDic = dic_dir)
    }
  }
}

#' @noRd
#' @param libpath libpath
.onUnload <- function(libpath) {
  library.dynam.unload("RcppMeCab", libpath)
}

#' Download and install a MeCab dictionary
#'
#' Downloads and installs a MeCab system dictionary for the specified language.
#' Japanese and Chinese dictionaries are compiled from source using the built-in
#' \code{mecab-dict-index}; Korean dictionaries are downloaded pre-compiled.
#' No system-level MeCab installation is required.
#'
#' Installing a dictionary does not change the MeCab engine linked into
#' RcppMeCab. Japanese and Chinese dictionaries use standard MeCab. Supported
#' Korean behavior requires the mecab-ko engine selected at package installation.
#'
#' Dictionaries are stored in the user data directory
#' (\code{tools::R_user_dir("RcppMeCab", "data")}).
#'
#' @param lang Character scalar. Dictionary code: \code{"ja"} for Japanese
#'   (IPAdic), \code{"ko"} for Korean (mecab-ko-dic), or \code{"zh"} for
#'   Chinese (mecab-jieba).
#' @return Invisible path to the installed dictionary directory.
#'
#' @examples
#' \dontrun{
#' download_dic("ja")
#' download_dic("ko")
#' download_dic("zh")
#' pos("some text", lang = "ja")
#' }
#'
#' @export
download_dic <- function(lang) {
  lang <- match.arg(lang, c("ja", "ko", "zh"))
  dic_dir <- file.path(tools::R_user_dir("RcppMeCab", "data"), lang)

  if (lang == "ja") {
    .download_dic_ja(dic_dir)
  } else if (lang == "ko") {
    .download_dic_ko(dic_dir)
  } else {
    .download_dic_zh(dic_dir)
  }

  message("Dictionary installed: ", dic_dir)
  invisible(dic_dir)
}

#' List installed MeCab dictionaries
#'
#' Shows all available MeCab dictionaries, including the bundled dictionary
#' and any downloaded via \code{\link{download_dic}}.
#'
#' @return A data frame with columns \code{lang}, \code{name}, \code{path},
#'   and \code{active}.
#'
#' @examples
#' \dontrun{
#' list_dic()
#' }
#'
#' @export
list_dic <- function() {
  user_dir <- tools::R_user_dir("RcppMeCab", "data")
  bundled <- system.file("dic", package = "RcppMeCab")
  active <- getOption("mecabSysDic", "")

  rows <- list()

  # Bundled dictionary
  if (nzchar(bundled) && file.exists(file.path(bundled, "sys.dic"))) {
    rows <- c(rows, list(data.frame(
      lang = "bundled", name = "bundled",
      path = bundled, active = (active == bundled),
      stringsAsFactors = FALSE
    )))
  }

  # User-installed dictionaries
  dic_names <- c(ja = "ipadic", ko = "mecab-ko-dic", zh = "mecab-jieba")
  for (lang in c("ja", "ko", "zh")) {
    dic_path <- file.path(user_dir, lang)
    if (file.exists(file.path(dic_path, "sys.dic"))) {
      name <- dic_names[[lang]]
      rows <- c(rows, list(data.frame(
        lang = lang, name = name,
        path = dic_path, active = (active == dic_path),
        stringsAsFactors = FALSE
      )))
    }
  }

  if (length(rows) == 0) {
    message("No dictionaries installed. Use download_dic() to install one.")
    return(invisible(data.frame(
      lang = character(), name = character(),
      path = character(), active = logical(),
      stringsAsFactors = FALSE
    )))
  }

  do.call(rbind, rows)
}

#' Set the active MeCab dictionary
#'
#' Sets the default system dictionary used by \code{\link{pos}} and
#' \code{\link{posParallel}}. This is equivalent to calling
#' \code{options(mecabSysDic = path)} but allows selection by dictionary code.
#' It does not switch the MeCab engine linked into RcppMeCab. Japanese and
#' Chinese dictionaries use standard MeCab; supported Korean behavior requires
#' the mecab-ko engine selected at package installation.
#'
#' @param lang Character scalar. Dictionary code (\code{"ja"}, \code{"ko"}, or
#'   \code{"zh"}) or \code{"bundled"} to use the dictionary bundled with the
#'   package.
#' @return Invisible path to the activated dictionary directory.
#'
#' @examples
#' \dontrun{
#' set_dic("ja")
#' pos("some Japanese text")
#'
#' set_dic("zh")
#' pos("some Chinese text")
#' }
#'
#' @export
set_dic <- function(lang) {
  lang <- match.arg(lang, c("ja", "ko", "zh", "bundled"))
  dic_path <- .resolve_dic(lang)
  options(mecabSysDic = dic_path)
  invisible(dic_path)
}

# ---- internal helpers --------------------------------------------------------

#' Resolve a dictionary code to a dictionary path
#' @noRd
.resolve_dic <- function(lang) {
  if (lang == "bundled") {
    dic_path <- system.file("dic", package = "RcppMeCab")
    if (!nzchar(dic_path) || !file.exists(file.path(dic_path, "sys.dic")))
      stop("No bundled dictionary found")
    return(dic_path)
  }

  dic_path <- file.path(tools::R_user_dir("RcppMeCab", "data"), lang)
  if (!file.exists(file.path(dic_path, "sys.dic")))
    stop("Dictionary for '", lang, "' not installed. Run download_dic(\"", lang, "\")")
  dic_path
}

#' Download and compile Japanese IPAdic
#' @noRd
.download_dic_ja <- function(dic_dir) {
  src_url <- "https://github.com/taku910/mecab/archive/61b90ba6e669dc2d7d533d4a80d206f3b31d52b1.tar.gz"
  tmp_dir <- tempfile("mecab_ja_")
  dir.create(tmp_dir, recursive = TRUE)
  on.exit(unlink(tmp_dir, recursive = TRUE), add = TRUE)

  tarball <- file.path(tmp_dir, "mecab.tar.gz")
  message("Downloading IPAdic source...")
  utils::download.file(src_url, tarball, mode = "wb", quiet = TRUE)

  message("Extracting...")
  utils::untar(tarball, exdir = tmp_dir)
  # taku910/mecab archive has mecab-<hash>/mecab-ipadic/ structure
  extracted <- list.dirs(tmp_dir, recursive = FALSE)
  ipadic_dir <- file.path(extracted[1], "mecab-ipadic")
  if (!dir.exists(ipadic_dir))
    stop("Could not find mecab-ipadic directory in archive")

  dir.create(dic_dir, recursive = TRUE, showWarnings = FALSE)

  message("Compiling dictionary (this may take a moment)...")
  args <- c("mecab-dict-index",
            "-d", normalizePath(ipadic_dir, mustWork = TRUE),
            "-o", normalizePath(dic_dir, mustWork = TRUE),
            "-f", "euc-jp",
            "-t", "utf-8")
  result <- dictIndexRcpp(args)
  if (result != 0)
    stop("Dictionary compilation failed (return code: ", result, ")")

  file.copy(file.path(ipadic_dir, "dicrc"), dic_dir, overwrite = TRUE)
  message("Japanese (IPAdic) dictionary installed.")
}

#' Download pre-compiled Korean mecab-ko-dic
#' @noRd
.download_dic_ko <- function(dic_dir) {
  dic_url <- "https://github.com/Pusnow/mecab-ko-msvc/releases/download/release-0.999/mecab-ko-dic.tar.gz"
  tmp_dir <- tempfile("mecab_ko_")
  dir.create(tmp_dir, recursive = TRUE)
  on.exit(unlink(tmp_dir, recursive = TRUE), add = TRUE)

  tarball <- file.path(tmp_dir, "mecab-ko-dic.tar.gz")
  message("Downloading mecab-ko-dic...")
  utils::download.file(dic_url, tarball, mode = "wb", quiet = TRUE)

  dir.create(dic_dir, recursive = TRUE, showWarnings = FALSE)

  message("Extracting...")
  utils::untar(tarball, exdir = dic_dir, extras = "--strip-components=1")
  if (!file.exists(file.path(dic_dir, "sys.dic")))
    stop("Dictionary extraction failed: sys.dic not found")

  message("Korean (mecab-ko-dic) dictionary installed.")
}

#' Download and compile Chinese mecab-jieba
#' @noRd
.download_dic_zh <- function(dic_dir) {
  src_url <- "https://github.com/lindera/mecab-jieba/archive/refs/tags/0.1.1.tar.gz"
  tmp_dir <- tempfile("mecab_zh_")
  dir.create(tmp_dir, recursive = TRUE)
  on.exit(unlink(tmp_dir, recursive = TRUE), add = TRUE)

  tarball <- file.path(tmp_dir, "mecab-jieba.tar.gz")
  message("Downloading mecab-jieba source...")
  utils::download.file(src_url, tarball, mode = "wb", quiet = TRUE)

  message("Extracting...")
  utils::untar(tarball, exdir = tmp_dir)
  extracted <- list.dirs(tmp_dir, recursive = FALSE)
  jieba_dir <- extracted[1]
  if (!file.exists(file.path(jieba_dir, "jieba.csv")))
    stop("Could not find jieba.csv in archive")

  dir.create(dic_dir, recursive = TRUE, showWarnings = FALSE)

  message("Compiling dictionary (this may take a moment)...")
  args <- c("mecab-dict-index",
            "-d", normalizePath(jieba_dir, mustWork = TRUE),
            "-o", normalizePath(dic_dir, mustWork = TRUE),
            "-f", "utf-8",
            "-t", "utf-8")
  result <- dictIndexRcpp(args)
  if (result != 0)
    stop("Dictionary compilation failed (return code: ", result, ")")

  file.copy(file.path(jieba_dir, "dicrc"), dic_dir, overwrite = TRUE)
  message("Chinese (mecab-jieba) dictionary installed.")
}

#' parallel version of part-of-speech tagger
#'
#' \code{posParallel} returns part-of-speech (POS) tagged morpheme of the sentence.
#'
#' This is a parallelized version of MeCab part-of-speech tagger. The function gets a
#' character vector of any length and runs a loop inside C++ with Intel TBB to provide faster
#' processing.
#'
#' Parallelizing over a character vector is not supported by \code{RcppParallel}.
#' Thus, this function makes duplicates of the input and the output.
#' Therefore, if your data volume is large, use \code{pos} or divide the vector to
#' several sub-vectors.
#'
#' You can add a user dictionary to \code{user_dic}. It should be compiled by
#' \code{mecab-dict-index}. You can find an explanation about compiling a user
#' dictionary in the \url{https://github.com/junhewk/RcppMeCab}.
#'
#' You can also set a system dictionary especially if you are using multiple
#' dictionaries (for example, using both IPA and Juman dictionary at the same time in Japanese)
#' in \code{sys_dic}. Using \code{options(mecabSysDic=)}, you can set your
#' preferred system dictionary to the R terminal.
#'
#' The \code{lang} argument selects a dictionary; it does not switch the MeCab
#' engine chosen when RcppMeCab was installed. Japanese and Chinese dictionaries
#' use standard MeCab. Supported Korean behavior, including mecab-ko whitespace
#' handling, requires the mecab-ko engine. Dictionary feature layouts are
#' language-specific, and the historical data-frame columns do not expose all
#' mecab-jieba metadata.
#'
#' If you want to get a morpheme only, use \code{join = False} to put tag names on the attribute.
#' Basically, the function will return a list of character vectors with (morpheme)/(tag) elements.
#'
#' @param sentence A character vector of any length. For analyzing multiple sentences, put them in one character vector.
#' @param join A bool to decide the output format. The default value is TRUE. If FALSE, the function will return morphemes only, and tags put in the attribute. if \code{format="data.frame"}, then this will be ignored.
#' @param format A data type for the result. The default value is "list". You can set this to "data.frame" to get a result as data frame format.
#' @param lang Optional dictionary code (\code{"ja"}, \code{"ko"}, or
#'   \code{"zh"}) selecting a dictionary installed via
#'   \code{\link{download_dic}}. This does not switch the MeCab engine. When
#'   specified, it overrides \code{sys_dic}.
#' @param sys_dic A location of system MeCab dictionary. The default value is "".
#' @param user_dic A location of user-specific MeCab dictionary. The default value is "".
#' @return A string vector or a list of POS tagged morpheme will be returned in conjoined character
#'  vector form.
#'
#' @examples
#' \dontrun{
#' sentence <- c(#some UTF-8 texts)
#' posParallel(sentence)
#' posParallel(sentence, join = FALSE)
#' posParallel(sentence, format = "data.frame")
#' posParallel(sentence, lang = "ja")
#' posParallel(sentence, lang = "zh")
#' posParallel(sentence, sys_dic = "/path/to/custom/dic")
#' posParallel(sentence, user_dic = "/path/to/user.dic")
#' }
#'
#' @export
posParallel <- function(sentence, join = TRUE, format = c("list", "data.frame"), lang = NULL, sys_dic = "", user_dic = "") {
  if (typeof(sentence) != "character") {
    if (typeof(sentence) == "factor") {
      stop("The type of input sentence is a factor. Please typesetting it with as.character().")
    } else {
      stop("The function gets a character vector only.")
    }
  }

  if (!is.null(lang)) {
    sys_dic <- .resolve_dic(match.arg(lang, c("ja", "ko", "zh")))
  } else if (!is.null(getOption("mecabSysDic")) && sys_dic == "") {
    sys_dic <- getOption("mecabSysDic")
  }

  format = match.arg(format)

  if (format == "data.frame") {
    result <- posParallelDFRcpp(sentence, sys_dic, user_dic)
  } else {
    if (join == TRUE) {
      result <- posParallelJoinRcpp(sentence, sys_dic, user_dic)
    } else {
      result <- posParallelRcpp(sentence, sys_dic, user_dic)
    }
  }

  return(result)
}

test_that("Chinese profile uses the bundled mecab-jieba dictionary", {
  skip_if_not(Sys.getenv("MECAB_LANG") == "zh", "Skip testing on Chinese")
  skip_if(!l10n_info()[["UTF-8"]], "Non-UTF-8 locale")

  simplified <- enc2utf8("\u6b66\u6c49\u5e02\u89e3\u9664\u79bb\u6c49\u79bb\u9102\u901a\u9053\u7ba1\u63a7\u63aa\u65bd")
  traditional <- enc2utf8("\u8fd1\u671f\u81ea\u716e\u9632\u75ab\u5df2\u6210\u4e86\u6700\u65b0\u98f2\u98df\u89c0")

  simplified_result <- pos(simplified, format = "data.frame")
  traditional_result <- pos(traditional, format = "data.frame")

  expect_gt(nrow(simplified_result), 0L)
  expect_gt(nrow(traditional_result), 0L)
  expect_identical(paste0(simplified_result$token, collapse = ""), simplified)
  expect_identical(paste0(traditional_result$token, collapse = ""), traditional)
  expect_true(enc2utf8("\u6b66\u6c49\u5e02") %in% simplified_result$token)
  expect_true(all(nzchar(simplified_result$pos)))

  parallel_result <- posParallel(simplified, format = "data.frame")
  expect_identical(parallel_result$token, simplified_result$token)
  expect_identical(parallel_result$pos, simplified_result$pos)

  info <- dictionary_info()
  expect_true(any(info$type == "system"))
  expect_true(all(toupper(info$charset) == "UTF-8"))
})

#' Compile a MeCab user dictionary
#'
#' \code{dict_index} compiles a user dictionary CSV file into a binary
#' dictionary that can be used with \code{pos} and \code{posParallel}.
#'
#' This function wraps MeCab's \code{mecab-dict-index} internally, so you
#' do not need the command-line tool installed separately.
#'
#' @param dic_csv Character scalar. Path to the user dictionary CSV file(s).
#'   Multiple CSV files can be provided as a character vector.
#' @param out_dic Character scalar. Path for the output compiled dictionary file.
#' @param dic_dir Character scalar. Path to the system dictionary directory.
#'   This is required so that MeCab can reference the system dictionary
#'   configuration during compilation.
#' @param dic_charset Character scalar. Charset of the input CSV file.
#'   Default is \code{"utf-8"}.
#' @param out_charset Character scalar. Charset of the output dictionary.
#'   Default is \code{"utf-8"}.
#' @return Invisible \code{TRUE} on success.
#'
#' @examples
#' \dontrun{
#' dict_index(
#'   dic_csv = "user_words.csv",
#'   out_dic = "user.dic",
#'   dic_dir = "/usr/local/lib/mecab/dic/ipadic"
#' )
#'
#' # Then use the compiled dictionary:
#' pos("some text", user_dic = "user.dic")
#' }
#'
#' @export
dict_index <- function(dic_csv, out_dic, dic_dir,
                       dic_charset = "utf-8", out_charset = "utf-8") {
  if (!is.character(dic_csv) || length(dic_csv) < 1)
    stop("dic_csv must be a character vector of CSV file path(s)")
  if (!is.character(out_dic) || length(out_dic) != 1)
    stop("out_dic must be a single output file path")
  if (!is.character(dic_dir) || length(dic_dir) != 1)
    stop("dic_dir must be a single directory path")

  for (f in dic_csv) {
    if (!file.exists(f))
      stop("CSV file not found: ", f)
  }
  if (!dir.exists(dic_dir))
    stop("System dictionary directory not found: ", dic_dir)

  out_dir <- dirname(out_dic)
  if (!dir.exists(out_dir))
    stop("Output directory does not exist: ", out_dir)

  args <- c("mecab-dict-index",
            "-d", normalizePath(dic_dir, mustWork = TRUE),
            "-u", normalizePath(out_dic, mustWork = FALSE),
            "-f", dic_charset,
            "-t", out_charset,
            normalizePath(dic_csv, mustWork = TRUE))

  dictIndexRcpp(args)

  invisible(TRUE)
}

#' RcppMeCab: Rcpp Wrapper for MeCab Library
#'
#' R package based on \code{Rcpp} for \code{MeCab}: Yet Another Part-of-Speech and
#' Morphological Analyzer (\url{http://taku910.github.io/mecab/}). It provides
#' install-time engine profiles and dictionaries for Japanese, Korean, and
#' Mandarin Chinese text. Runtime dictionary selection does not change the
#' installed engine. This package utilizes parallel programming for providing
#' highly efficient text preprocessing \code{posParallel()} function.
#' For installation, please refer to README.md file.
#'
#' @name RcppMeCab
#' @docType package
#' @author Junhewk Kim Taku Kudo
#' @import Rcpp
#' @importFrom RcppParallel RcppParallelLibs
#' @details
#' This package utilizes \code{MeCab} C API and \code{Rcpp} codes.
#'
#' @references
#' \itemize{
#' \item{\href{http://taku910.github.io/mecab/}{MeCab}}
#' \item{\href{https://www.rcpp.org/}{Rcpp: Seamless R and C++ Integration}}
#' \item{\href{http://eunjeon.blogspot.com}{Eunjeon project}}
#' }
#'
#' @keywords MeCab part-of-speech morpheme nlp Korean Japanese Chinese
"_PACKAGE"

#' Inspect loaded MeCab dictionaries
#'
#' Returns metadata reported by MeCab for the active system and user
#' dictionaries. This is useful for diagnosing which dictionary a package or
#' R session is actually using.
#'
#' @param sys_dic Character scalar. System dictionary directory. When empty,
#'   the `mecabSysDic` option and then MeCab's default are used.
#' @param user_dic Character scalar. Optional compiled user dictionary.
#' @return A data frame with one row per loaded dictionary and columns
#'   `filename`, `charset`, `type`, `size`, `left_size`, `right_size`, and
#'   `version`.
#' @examples
#' \dontrun{
#' dictionary_info()
#' }
#' @export
dictionary_info <- function(sys_dic = "", user_dic = "") {
  if (!is.character(sys_dic) || length(sys_dic) != 1L || is.na(sys_dic)) {
    stop("sys_dic must be a non-missing character scalar", call. = FALSE)
  }
  if (!is.character(user_dic) || length(user_dic) != 1L || is.na(user_dic)) {
    stop("user_dic must be a non-missing character scalar", call. = FALSE)
  }

  if (!nzchar(sys_dic) && !is.null(getOption("mecabSysDic"))) {
    sys_dic <- getOption("mecabSysDic")
  }

  dictionaryInfoRcpp(sys_dic, user_dic)
}

test_that("dictionary_info reports the loaded system dictionary", {
  skip_if(Sys.info()[["sysname"]] == "Windows" &&
            identical(Sys.getenv("MECAB_LANG"), "ja"),
          "Japanese dictionary is not bundled on Windows")
  info <- tryCatch(dictionary_info(), error = function(e) NULL)
  skip_if(is.null(info), "MeCab dictionary not available")

  expect_s3_class(info, "data.frame")
  expect_named(
    info,
    c("filename", "charset", "type", "size", "left_size", "right_size", "version")
  )
  expect_gte(nrow(info), 1L)
  expect_true(any(info$type == "system"))
  expect_true(all(nzchar(info$filename)))
})

test_that("dictionary_info validates arguments", {
  expect_error(dictionary_info(NA_character_), "sys_dic")
  expect_error(dictionary_info(user_dic = character()), "user_dic")
})

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

dictIndexRcpp <- function(args) {
    .Call(`_RcppMeCab_dictIndexRcpp`, args)
}

dictionaryInfoRcpp <- function(sys_dic, user_dic) {
    .Call(`_RcppMeCab_dictionaryInfoRcpp`, sys_dic, user_dic)
}

posParallelJoinRcpp <- function(text, sys_dic, user_dic) {
    .Call(`_RcppMeCab_posParallelJoinRcpp`, text, sys_dic, user_dic)
}

posParallelDFRcpp <- function(text, sys_dic, user_dic) {
    .Call(`_RcppMeCab_posParallelDFRcpp`, text, sys_dic, user_dic)
}

posParallelRcpp <- function(text, sys_dic, user_dic) {
    .Call(`_RcppMeCab_posParallelRcpp`, text, sys_dic, user_dic)
}

posRcpp <- function(text, sys_dic, user_dic) {
    .Call(`_RcppMeCab_posRcpp`, text, sys_dic, user_dic)
}

posJoinRcpp <- function(text, sys_dic, user_dic) {
    .Call(`_RcppMeCab_posJoinRcpp`, text, sys_dic, user_dic)
}

posLoopRcpp <- function(text, sys_dic, user_dic) {
    .Call(`_RcppMeCab_posLoopRcpp`, text, sys_dic, user_dic)
}

posLoopJoinRcpp <- function(text, sys_dic, user_dic) {
    .Call(`_RcppMeCab_posLoopJoinRcpp`, text, sys_dic, user_dic)
}

posDFRcpp <- function(text, sys_dic, user_dic) {
    .Call(`_RcppMeCab_posDFRcpp`, text, sys_dic, user_dic)
}

# Register entry points for exported C++ functions
methods::setLoadAction(function(ns) {
    .Call(`_RcppMeCab_RcppExport_registerCCallable`)
})

# Download and patch MeCab source for Windows build
# Selects taku910/mecab for Japanese/Chinese or Pusnow/mecab-ko-msvc for Korean
# based on the MECAB_LANG environment variable (default: "ko").

MECAB_LANG <- Sys.getenv("MECAB_LANG", "ko")
if (!MECAB_LANG %in% c("ja", "ko", "zh")) {
  stop("Unsupported MECAB_LANG='", MECAB_LANG,
       "'. Expected one of: ja, ko, zh.")
}

if (MECAB_LANG %in% c("ja", "zh")) {
  MECAB_SRC_URL <- "https://github.com/taku910/mecab/archive/61b90ba6e669dc2d7d533d4a80d206f3b31d52b1.tar.gz"
  MECAB_VERSION <- "0.996"
} else {
  MECAB_SRC_URL <- "https://github.com/Pusnow/mecab-ko-msvc/archive/refs/tags/release-0.999.tar.gz"
  MECAB_VERSION <- "0.999"
}

BUILD_DIR <- "mecab_build"
SRC_DIR <- file.path(BUILD_DIR, "mecab", "src")

if (!dir.exists(BUILD_DIR)) {
  dir.create(BUILD_DIR, recursive = TRUE)

  # Download
  tarball <- file.path(BUILD_DIR, "mecab-source.tar.gz")
  download.file(MECAB_SRC_URL, tarball, mode = "wb", method = "libcurl")
  untar(tarball, exdir = BUILD_DIR, extras = "--strip-components=1")

  # For mecab-ko-msvc, source is in src/ directly (not mecab/src/)
  if (MECAB_LANG == "ko" && !dir.exists(SRC_DIR)) {
    dir.create(file.path(BUILD_DIR, "mecab"), recursive = TRUE)
    file.rename(file.path(BUILD_DIR, "src"), SRC_DIR)
  }

  cat("Using MeCab source:", MECAB_LANG, MECAB_VERSION, "\n")

  # Write R-compat header
  writeLines(c(
    "#ifndef MECAB_R_COMPAT_H_",
    "#define MECAB_R_COMPAT_H_",
    "#include <iostream>",
    "#include <cstdio>",
    "class MeCabNullBuf : public std::streambuf {",
    "protected:",
    "  int overflow(int c) { return c; }",
    "};",
    "static MeCabNullBuf mecab_null_buf_;",
    "static std::ostream mecab_null_os_(&mecab_null_buf_);",
    "#endif"
  ), file.path(SRC_DIR, "mecab_r_compat.h"))

  # Write config.h
  writeLines(c(
    "#define HAVE_GETENV 1",
    "#define HAVE_STRING_H 1",
    "#define HAVE_STDLIB_H 1",
    "#define HAVE_STDINT_H 1",
    "#define HAVE_WINDOWS_H 1",
    paste0("#define PACKAGE \"mecab\""),
    paste0("#define VERSION \"", MECAB_VERSION, "\""),
    "#define DIC_VERSION 102"
  ), file.path(SRC_DIR, "config.h"))

  # Patch source files: prepend R-compat header, redirect cerr/cout
  src_files <- list.files(SRC_DIR, pattern = "\\.(cpp|h)$", full.names = TRUE)
  src_files <- src_files[!grepl("mecab_r_compat\\.h|config\\.h", src_files)]

  for (f in src_files) {
    lines <- readLines(f, warn = FALSE)
    lines <- c('#include "mecab_r_compat.h"', lines)
    lines <- gsub("std::cerr", "mecab_null_os_", lines)
    lines <- gsub("std::cout", "mecab_null_os_", lines)
    writeLines(lines, f)
  }

  # mecab.h: disable dllimport for static linking
  f <- file.path(SRC_DIR, "mecab.h")
  lines <- readLines(f, warn = FALSE)
  lines <- gsub("__declspec\\(dllimport\\)", "extern", lines)
  writeLines(lines, f)

  # thread.h: fix _stdcall -> __stdcall for MinGW
  f <- file.path(SRC_DIR, "thread.h")
  lines <- readLines(f, warn = FALSE)
  lines <- gsub("_stdcall", "__stdcall", lines, fixed = TRUE)
  writeLines(lines, f)

  # string_buffer.h: add size_t overload for 64-bit MinGW (size_t = unsigned long long)
  f <- file.path(SRC_DIR, "string_buffer.h")
  lines <- readLines(f, warn = FALSE)
  lines <- gsub("(operator<<\\(unsigned long int n\\)\\s*\\{[^}]*\\})",
                "\\1\n  StringBuffer& operator<<(unsigned long long int n) { _UITOA(n); }",
                lines)
  writeLines(lines, f)

  # common.h: remove exit(-1) from die class
  f <- file.path(SRC_DIR, "common.h")
  lines <- readLines(f, warn = FALSE)
  lines <- lines[!grepl("^\\s*exit\\(-1\\);", lines)]
  writeLines(lines, f)

  # utils.h: sprintf -> snprintf
  f <- file.path(SRC_DIR, "utils.h")
  lines <- readLines(f, warn = FALSE)
  lines <- gsub('std::sprintf\\(s, "%-16f", val\\)',
                'std::snprintf(s, sizeof(s), "%-16f", val)', lines)
  writeLines(lines, f)

  # utils.cpp: gut progress_bar body
  f <- file.path(SRC_DIR, "utils.cpp")
  lines <- readLines(f, warn = FALSE)
  in_func <- FALSE
  brace_depth <- 0
  new_lines <- character()
  for (line in lines) {
    if (!in_func && grepl("^int progress_bar", line)) {
      in_func <- TRUE
      brace_depth <- brace_depth + lengths(regmatches(line, gregexpr("\\{", line)))
      brace_depth <- brace_depth - lengths(regmatches(line, gregexpr("\\}", line)))
      new_lines <- c(new_lines, line)
      next
    }
    if (in_func) {
      brace_depth <- brace_depth + lengths(regmatches(line, gregexpr("\\{", line)))
      brace_depth <- brace_depth - lengths(regmatches(line, gregexpr("\\}", line)))
      if (brace_depth <= 0 && grepl("\\}", line)) {
        new_lines <- c(new_lines, "  return 1;", "}")
        in_func <- FALSE
      }
      next
    }
    new_lines <- c(new_lines, line)
  }
  writeLines(new_lines, f)

  # eval.cpp: sprintf -> snprintf
  f <- file.path(SRC_DIR, "eval.cpp")
  lines <- readLines(f, warn = FALSE)
  lines <- gsub("sprintf\\(buf\\.get\\(\\),", "snprintf(buf.get(), 256,", lines)
  writeLines(lines, f)

  cat("MeCab source patched successfully.\n")
} else {
  cat("MeCab source already prepared.\n")
}

# Download and install dictionary into inst/dic/
DIC_DIR <- file.path("..", "inst", "dic")
if (!file.exists(file.path(DIC_DIR, "sys.dic"))) {
  dir.create(DIC_DIR, recursive = TRUE)

  if (MECAB_LANG == "ja") {
    cat("Compiling IPAdic (Japanese) dictionary...\n")
    # ipadic source is in the same taku910/mecab archive
    IPADIC_DIR <- file.path(BUILD_DIR, "mecab-ipadic")
    if (!dir.exists(IPADIC_DIR)) {
      # Re-download if needed
      tarball <- file.path(BUILD_DIR, "mecab-source.tar.gz")
      if (!file.exists(tarball)) {
        download.file(MECAB_SRC_URL, tarball, mode = "wb", method = "libcurl")
        untar(tarball, exdir = BUILD_DIR, extras = "--strip-components=1")
      }
    }
    # Use mecab-dict-index from the built MeCab source to compile ipadic
    # On Windows, we compile dictionary_compiler.cpp as part of the build,
    # but mecab-dict-index is not a standalone binary. Instead, download
    # pre-compiled ipadic from a known source.
    # For now, copy the source .def files and compile after package install.
    # Alternative: skip dictionary for ja on Windows if no pre-compiled available.
    cat("WARNING: Japanese dictionary compilation on Windows not yet supported during install.\n")
    cat("Please install ipadic manually.\n")
  } else if (MECAB_LANG == "zh") {
    # The Makevars.win zh target compiles this source after the MeCab objects
    # are available, using a temporary mecab-dict-index executable.
    JIEBA_URL <- "https://github.com/lindera/mecab-jieba/archive/refs/tags/0.1.1.tar.gz"
    JIEBA_DIR <- file.path(BUILD_DIR, "mecab-jieba-src")
    jieba_tarball <- file.path(BUILD_DIR, "mecab-jieba.tar.gz")
    cat("Downloading mecab-jieba 0.1.1 source...\n")
    download.file(JIEBA_URL, jieba_tarball, mode = "wb", method = "libcurl")
    dir.create(JIEBA_DIR, recursive = TRUE)
    untar(jieba_tarball, exdir = JIEBA_DIR, extras = "--strip-components=1")
    if (!file.exists(file.path(JIEBA_DIR, "jieba.csv"))) {
      stop("Could not find jieba.csv in mecab-jieba archive")
    }
    cat("Chinese dictionary source staged at:", JIEBA_DIR, "\n")
  } else {
    cat("Downloading pre-compiled mecab-ko-dic...\n")
    DIC_URL <- "https://github.com/Pusnow/mecab-ko-msvc/releases/download/release-0.999/mecab-ko-dic.tar.gz"
    dic_tarball <- file.path(BUILD_DIR, "mecab-ko-dic.tar.gz")
    tryCatch({
      download.file(DIC_URL, dic_tarball, mode = "wb", method = "libcurl")
      untar(dic_tarball, exdir = DIC_DIR, extras = "--strip-components=1")
      cat("mecab-ko-dic installed to:", DIC_DIR, "\n")
    }, error = function(e) {
      cat("WARNING: Failed to download mecab-ko-dic:", conditionMessage(e), "\n")
      cat("Package will work but needs manual dictionary setup.\n")
    })
  }
} else {
  cat("Dictionary already installed.\n")
}

test_that("Test pos tagger works on ko", {
  skip_if(.Platform$OS.type == "windows", "Skip testing on Windows")
  skip_if(!l10n_info()[["UTF-8"]], "Non-UTF-8 locale")
  skip_if_not(Sys.getenv("MECAB_LANG", "ko") == "ko", "Skip testing on Korean")
  result <- tryCatch(
    pos(
      enc2utf8("\uc548\ub155\ud558\uc138\uc694 \uc800\ub294 \ubc15\ucc2c\uc5fd \uc785\ub2c8\ub2e4."),
      format = "data.frame"
    ),
    error = function(e) NULL
  )
  skip_if(is.null(result) || nrow(result) == 0, "Korean dictionary not available")
  expect_equal(result[4, 4], enc2utf8("\uc800"))
})

test_that("Test pos tagger works on ja", {
  skip_if(.Platform$OS.type == "windows", "Skip testing on Windows")
  skip_if(!l10n_info()[["UTF-8"]], "Non-UTF-8 locale")
  skip_if_not(Sys.getenv("MECAB_LANG") == "ja", "Skip testing on Japanese")
  result <- tryCatch(
    pos(
      enc2utf8("\u982d\u304c\u8d64\u3044\u9b5a\u3092\u98df\u3079\u305f\u732b"),
      format = "data.frame"
    ),
    error = function(e) NULL
  )
  skip_if(is.null(result) || nrow(result) == 0, "Japanese dictionary not available")
  expect_equal(result[4, 4], enc2utf8("\u9b5a"))
})

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppMeCab-package.r
\docType{package}
\name{RcppMeCab}
\alias{RcppMeCab-package}
\alias{RcppMeCab}
\title{RcppMeCab: Rcpp Wrapper for MeCab Library}
\description{
R package based on \code{Rcpp} for \code{MeCab}: Yet Another Part-of-Speech and
Morphological Analyzer (\url{http://taku910.github.io/mecab/}). It provides
install-time engine profiles and dictionaries for Japanese, Korean, and
Mandarin Chinese text. Runtime dictionary selection does not change the
installed engine. This package utilizes parallel programming for providing
highly efficient text preprocessing \code{posParallel()} function.
For installation, please refer to README.md file.
}
\details{
This package utilizes \code{MeCab} C API and \code{Rcpp} codes.
}
\references{
\itemize{
\item{\href{http://taku910.github.io/mecab/}{MeCab}}
\item{\href{https://www.rcpp.org/}{Rcpp: Seamless R and C++ Integration}}
\item{\href{http://eunjeon.blogspot.com}{Eunjeon project}}
}
}
\seealso{
Useful links:
\itemize{
  \item Report bugs at \url{https://github.com/junhewk/RcppMeCab/issues}
}

}
\author{
Junhewk Kim Taku Kudo
}
\keyword{Chinese}
\keyword{Japanese}
\keyword{Korean}
\keyword{MeCab}
\keyword{morpheme}
\keyword{nlp}
\keyword{part-of-speech}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dic.R
\name{download_dic}
\alias{download_dic}
\title{Download and install a MeCab dictionary}
\usage{
download_dic(lang)
}
\arguments{
\item{lang}{Character scalar. Dictionary code: \code{"ja"} for Japanese
(IPAdic), \code{"ko"} for Korean (mecab-ko-dic), or \code{"zh"} for
Chinese (mecab-jieba).}
}
\value{
Invisible path to the installed dictionary directory.
}
\description{
Downloads and installs a MeCab system dictionary for the specified language.
Japanese and Chinese dictionaries are compiled from source using the built-in
\code{mecab-dict-index}; Korean dictionaries are downloaded pre-compiled.
No system-level MeCab installation is required.

Installing a dictionary does not change the MeCab engine linked into
RcppMeCab. Japanese and Chinese dictionaries use standard MeCab. Supported
Korean behavior requires the mecab-ko engine selected at package installation.
}
\details{
Dictionaries are stored in the user data directory
(\code{tools::R_user_dir("RcppMeCab", "data")}).
}
\examples{
\dontrun{
download_dic("ja")
download_dic("ko")
download_dic("zh")
pos("some text", lang = "ja")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dict.R
\name{dict_index}
\alias{dict_index}
\title{Compile a MeCab user dictionary}
\usage{
dict_index(
  dic_csv,
  out_dic,
  dic_dir,
  dic_charset = "utf-8",
  out_charset = "utf-8"
)
}
\arguments{
\item{dic_csv}{Character scalar. Path to the user dictionary CSV file(s).
Multiple CSV files can be provided as a character vector.}

\item{out_dic}{Character scalar. Path for the output compiled dictionary file.}

\item{dic_dir}{Character scalar. Path to the system dictionary directory.
This is required so that MeCab can reference the system dictionary
configuration during compilation.}

\item{dic_charset}{Character scalar. Charset of the input CSV file.
Default is \code{"utf-8"}.}

\item{out_charset}{Character scalar. Charset of the output dictionary.
Default is \code{"utf-8"}.}
}
\value{
Invisible \code{TRUE} on success.
}
\description{
\code{dict_index} compiles a user dictionary CSV file into a binary
dictionary that can be used with \code{pos} and \code{posParallel}.
}
\details{
This function wraps MeCab's \code{mecab-dict-index} internally, so you
do not need the command-line tool installed separately.
}
\examples{
\dontrun{
dict_index(
  dic_csv = "user_words.csv",
  out_dic = "user.dic",
  dic_dir = "/usr/local/lib/mecab/dic/ipadic"
)

# Then use the compiled dictionary:
pos("some text", user_dic = "user.dic")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dic.R
\name{list_dic}
\alias{list_dic}
\title{List installed MeCab dictionaries}
\usage{
list_dic()
}
\value{
A data frame with columns \code{lang}, \code{name}, \code{path},
  and \code{active}.
}
\description{
Shows all available MeCab dictionaries, including the bundled dictionary
and any downloaded via \code{\link{download_dic}}.
}
\examples{
\dontrun{
list_dic()
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/posParallel.R
\name{posParallel}
\alias{posParallel}
\title{parallel version of part-of-speech tagger}
\usage{
posParallel(
  sentence,
  join = TRUE,
  format = c("list", "data.frame"),
  lang = NULL,
  sys_dic = "",
  user_dic = ""
)
}
\arguments{
\item{sentence}{A character vector of any length. For analyzing multiple sentences, put them in one character vector.}

\item{join}{A bool to decide the output format. The default value is TRUE. If FALSE, the function will return morphemes only, and tags put in the attribute. if \code{format="data.frame"}, then this will be ignored.}

\item{format}{A data type for the result. The default value is "list". You can set this to "data.frame" to get a result as data frame format.}

\item{lang}{Optional dictionary code (\code{"ja"}, \code{"ko"}, or
\code{"zh"}) selecting a dictionary installed via
\code{\link{download_dic}}. This does not switch the MeCab engine. When
specified, it overrides \code{sys_dic}.}

\item{sys_dic}{A location of system MeCab dictionary. The default value is "".}

\item{user_dic}{A location of user-specific MeCab dictionary. The default value is "".}
}
\value{
A string vector or a list of POS tagged morpheme will be returned in conjoined character
 vector form.
}
\description{
\code{posParallel} returns part-of-speech (POS) tagged morpheme of the sentence.
}
\details{
This is a parallelized version of MeCab part-of-speech tagger. The function gets a
character vector of any length and runs a loop inside C++ with Intel TBB to provide faster
processing.

Parallelizing over a character vector is not supported by \code{RcppParallel}.
Thus, this function makes duplicates of the input and the output.
Therefore, if your data volume is large, use \code{pos} or divide the vector to
several sub-vectors.

You can add a user dictionary to \code{user_dic}. It should be compiled by
\code{mecab-dict-index}. You can find an explanation about compiling a user
dictionary in the \url{https://github.com/junhewk/RcppMeCab}.

You can also set a system dictionary especially if you are using multiple
dictionaries (for example, using both IPA and Juman dictionary at the same time in Japanese)
in \code{sys_dic}. Using \code{options(mecabSysDic=)}, you can set your
preferred system dictionary to the R terminal.

The \code{lang} argument selects a dictionary; it does not switch the MeCab
engine chosen when RcppMeCab was installed. Japanese and Chinese dictionaries
use standard MeCab. Supported Korean behavior, including mecab-ko whitespace
handling, requires the mecab-ko engine. Dictionary feature layouts are
language-specific, and the historical data-frame columns do not expose all
mecab-jieba metadata.

If you want to get a morpheme only, use \code{join = False} to put tag names on the attribute.
Basically, the function will return a list of character vectors with (morpheme)/(tag) elements.
}
\examples{
\dontrun{
sentence <- c(#some UTF-8 texts)
posParallel(sentence)
posParallel(sentence, join = FALSE)
posParallel(sentence, format = "data.frame")
posParallel(sentence, lang = "ja")
posParallel(sentence, lang = "zh")
posParallel(sentence, sys_dic = "/path/to/custom/dic")
posParallel(sentence, user_dic = "/path/to/user.dic")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dictionary.R
\name{dictionary_info}
\alias{dictionary_info}
\title{Inspect loaded MeCab dictionaries}
\usage{
dictionary_info(sys_dic = "", user_dic = "")
}
\arguments{
\item{sys_dic}{Character scalar. System dictionary directory. When empty,
the \code{mecabSysDic} option and then MeCab's default are used.}

\item{user_dic}{Character scalar. Optional compiled user dictionary.}
}
\value{
A data frame with one row per loaded dictionary and columns
\code{filename}, \code{charset}, \code{type}, \code{size}, \code{left_size},
\code{right_size}, and \code{version}.
}
\description{
Returns metadata reported by MeCab for the active system and user
dictionaries. This is useful for diagnosing which dictionary a package or
R session is actually using.
}
\examples{
\dontrun{
dictionary_info()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dic.R
\name{set_dic}
\alias{set_dic}
\title{Set the active MeCab dictionary}
\usage{
set_dic(lang)
}
\arguments{
\item{lang}{Character scalar. Dictionary code (\code{"ja"}, \code{"ko"}, or
\code{"zh"}) or \code{"bundled"} to use the dictionary bundled with the
package.}
}
\value{
Invisible path to the activated dictionary directory.
}
\description{
Sets the default system dictionary used by \code{\link{pos}} and
\code{\link{posParallel}}. This is equivalent to calling
\code{options(mecabSysDic = path)} but allows selection by dictionary code.
It does not switch the MeCab engine linked into RcppMeCab. Japanese and
Chinese dictionaries use standard MeCab; supported Korean behavior requires
the mecab-ko engine selected at package installation.
}
\examples{
\dontrun{
set_dic("ja")
pos("some Japanese text")

set_dic("zh")
pos("some Chinese text")
}

}
